#define FILELIST_H

/****************************************************************

    Copyright (c) 1996-1997 Fred Pesch

    This program is free software; you can redistribute it 
    and/or modify it under the terms of the GNU General 
    Public License as published by the Free Software 
    Foundation (version 2 of the License).

    This program is distributed in the hope that it will 
    be useful, but WITHOUT ANY WARRANTY; without even the 
    implied warranty of MERCHANTABILITY or FITNESS FOR A 
    PARTICULAR PURPOSE.  See the GNU General Public License 
    for more details.

    For a copy of the GNU General Public License, write to the 
    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
    MA 02139, USA.

****************************************************************/

/*** FOR DEBUGGING ONLY ***/
#include <stdio.h>
/**************************/


#include <stdlib.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/stat.h>
#include <Xm/Xm.h>

#include "dirlist.h"
#include "error_handler.h"
#include "config.h"

extern uid_t UserID;
extern gid_t GroupID;

typedef enum { DIRECTORY, SLINK_DIR, SLINK_FILE, EXECUTABLE_FILE, REGULAR_FILE, CHAR_DEVICE, BLOCK_DEVICE, FIFO, SOCKET, UNKNOWN } PRIMARY_FILE_TYPE;


typedef struct {

  SECURITY_FLAGS user;
  SECURITY_FLAGS group;
  SECURITY_FLAGS others;

  PRIMARY_FILE_TYPE type;

  int order;
  int readable;
  int executable;
  int selected;
  time_t modiftime;
  time_t changetime;
  time_t accesstime;
  off_t size;
  uid_t UID;
  gid_t GID;
  char *name;
} TFileEntry;

struct FileNode {
  TDirNode *parent;
  TFileEntry file;
  struct FileNode *next;
  struct FileNode *previous;
};

typedef struct FileNode TFileNode;

void CreateGoBackDirectory(TDirNode *parent, TFileNode *node)
{
  node->parent = parent;
  node->next = NULL;
  node->previous = NULL;

  node->file.user = parent->dir.user;
  node->file.group = parent->dir.group;
  node->file.others = parent->dir.others;
  
  node->file.type = DIRECTORY;

  node->file.order = 0;

  node->file.readable = parent->dir.readable;
  node->file.executable = 1;
  node->file.selected = 0;

  node->file.UID = parent->dir.UID;
  node->file.GID = parent->dir.GID;

  node->file.modiftime= parent->dir.modiftime;
  node->file.changetime = parent->dir.changetime;
  node->file.accesstime = parent->dir.accesstime;

  node->file.size = parent->dir.size;

  node->file.name=(char *) XtMalloc(sizeof(char)*3);
  if(node->file.name==NULL) FatalError("filelist.c","CreateGoBackDirectory","Unable to create file name (XtMalloc fails)");   
  strcpy(node->file.name,"..");
}

TFileNode *CreateFileEntry(struct stat *statbuf, TDirNode *parent, TFileNode *previous, char *name)
{
  TFileNode *node;
  umode_t mode;
  struct stat statbuf2;
  int result;

  node = (TFileNode *) XtMalloc(sizeof(TFileNode));
  if(node==NULL) FatalError("dirlist.c","CreateFileEntry","Unable to create File Node (XtMalloc failed)");

  node->parent = parent;
  node->next = NULL;
  node->previous = previous;

  mode = statbuf->st_mode;

  /* Get Name */
  node->file.name=(char *) XtMalloc(sizeof(char)*(strlen(name)+1));
  if(node->file.name==NULL) FatalError("filelist.c","CreateFileEntry","Unable to create file name (XtMalloc fails)"); 
  strcpy(node->file.name, name);

  /* Get UID */
  node->file.UID = statbuf->st_uid;
  
  /* Get GID */
  node->file.GID = statbuf->st_gid;

  /* Get Size */
  node->file.size = statbuf->st_size;

  /* Get Last Modification Time */
  node->file.modiftime = statbuf->st_mtime;

  /* Get Last Change time */
  node->file.changetime = statbuf->st_ctime;
    
  /* Get Last Access time */
  node->file.accesstime = statbuf->st_atime;

  /* Get User protections */
  node->file.user=NONE;

  if(mode & S_IRUSR) node->file.user |= READ;
  if(mode & S_IWUSR) node->file.user |= WRITE;
  if(mode & S_IXUSR) node->file.user |= EXECUTE;

  /* Get Group protections */
  node->file.group=NONE;

  if(mode & S_IRGRP) node->file.group |= READ;
  if(mode & S_IWGRP) node->file.group |= WRITE;
  if(mode & S_IXGRP) node->file.group |= EXECUTE;

  /* Get Others protections */
  node->file.others=NONE;

  if(mode & S_IROTH) node->file.others |= READ;
  if(mode & S_IWOTH) node->file.others |= WRITE;
  if(mode & S_IXOTH) node->file.others |= EXECUTE;

  /* default values */

  if(((node->file.user & READ) && (node->file.UID==UserID)) || 
     ((node->file.group & READ) &&(node->file.GID==GroupID)) ||
     (node->file.others & READ) ) 
    node->file.readable=1; 
  else node->file.readable=0;

  if(((node->file.user & EXECUTE) && (node->file.UID==UserID)) || 
     ((node->file.group & EXECUTE) &&(node->file.GID==GroupID)) ||
     (node->file.others & EXECUTE)) 
    node->file.executable=1; 
  else node->file.executable=0;  
  
  if(S_ISLNK(mode)) {
    result = stat(node->file.name, &statbuf2);
    if(S_ISDIR(statbuf2.st_mode)) node->file.type = SLINK_DIR;
    else node->file.type = SLINK_FILE;
  }
  else if(S_ISREG(mode)) {
    if(((node->file.user & EXECUTE) && (node->file.UID==UserID)) ||
       ((node->file.group & EXECUTE) && (node->file.GID==GroupID)) ||
       (node->file.others & EXECUTE)) node->file.type = EXECUTABLE_FILE;
    else node->file.type = REGULAR_FILE;
  }
  else if(S_ISDIR(mode)) node->file.type = DIRECTORY;
  else if(S_ISCHR(mode)) node->file.type = CHAR_DEVICE;
  else if(S_ISBLK(mode)) node->file.type = BLOCK_DEVICE;
  else if(S_ISFIFO(mode)) node->file.type = FIFO;
  else if(S_ISSOCK(mode)) node->file.type = SOCKET;
  else {
    node->file.type = UNKNOWN;
    NonFatalError("filelist.c","CreateFileEntry","Unknow primary file type");
  }

  node->file.selected = 0;

  node->file.order = previous->file.order + 1;

  return(node);
}

TFileNode *CreateFileList(TDirNode *parent)
{
  TFileNode *start;
  TFileNode *actual;
  DIR *dir;
  struct dirent *entry;
  int result;
  struct stat statbuf;
  Bool symlink_enabled;

  if(!parent->dir.readable) {
    NonFatalError("filelist.c","CreateFileList","Unable to open unreadable directory");
    return(NULL);
  }

  start = (TFileNode *) XtMalloc(sizeof(TFileNode));
  if(start==NULL) FatalError("filelist.c","CreateFileList","Unable to create File Node (XtMalloc failed)");
  
  CreateGoBackDirectory(parent, start);

  if(parent->parent==NULL) result = chdir("/");
  else result = chdir(parent->dir.pathname);
  if(result==-1) FatalError("filelist.c","CreateFileList","Cannot chdir to parent pathname (chdir failed)");

  dir = opendir(".");
  if(dir==NULL) FatalError("filelist.c","CreateFileList","Unable to open current directory (XtMalloc failed)");

  actual=start;

  do {
    entry = readdir(dir);
    if(entry!=NULL && (strcmp(entry->d_name,".")!=0) && (strcmp(entry->d_name,"..")!=0)) {
      if(GetVarBool("EnableSymbolicLinks",&symlink_enabled)) 
	if(symlink_enabled)
	  result = lstat(entry->d_name, &statbuf);
	else
	  result = stat(entry->d_name, &statbuf);
      if(result==-1) FatalError("filelist.c","MakeDirOneLevel","Cannot stat directory entry (stat failed)");
      actual->next = CreateFileEntry(&statbuf, parent, actual, entry->d_name);
      actual=actual->next;
    }
  } while(entry!=NULL);
  closedir(dir);

  return(start);
}

void DestroyFileList(TFileNode *start)
{
  TFileNode *destnode;

  while(start!=NULL) {
    destnode=start;
    start=start->next;
    XtFree(destnode->file.name);
    XtFree((char *)destnode);
  }
}
      









